!> 
!! \file client_miriade.f90
!<

!>
!! \brief Client programme to call the method 'ephemcc' of the webservice Miriade
!!
!! This program show how to consume the VO Miriade webservice to request a tabulated
!! ephemeris of Solar system objects. This example show how to retrieve ephemeris
!! of the Earth in the mean J2000 heliocentric reference frame.
!!
!! Miriade web page: http://vo.imcce.fr/webservices/miriade
!!
!! The program makes use of the network downloader 'wget', which can be substituted
!! by curl.
!!
!! \author IMCCE - J.Berthier <berthier@imcce.fr> 
!! \version 1.0
!! \date 2010-2016
!!
!! \par Compile:
!!  \c ifort -o client_miriade client_miriade.f90
!!
!! \par Usage:
!!  \c ./client_miriade
!<
program client_miriade

   implicit none
   character(len=32), dimension(12)   :: arg
   character(len=64)                  :: url, wget
   character(len=384)                 :: opt
   character(len=512)                 :: cmd
   integer                            :: i, err, errnum, nbDates
   integer(8)                         :: ticket

   character(len=14)                  :: RA, DEC
   character(len=22)                  :: epoch
   character(len=256), dimension(100) :: results
   real(kind=8)                       :: Dist, Mv

   ! Definition de wget ou curl
   wget    = 'wget -q -O ephemeris.dat'   ! alternative: 'curl -o "ephemeris.dat"'
   ! Service end point
   url     = 'http://vo.imcce.fr/webservices/miriade/ephemcc_query.php?'
   ! Defini les options
   arg(1)  = '-name=p:Earth'             ! or a:Ceres
   arg(2)  = '-ep=2010-11-18T12:24:08'   ! or 2455519.016759259
   arg(3)  = '-nbd=5'                    ! nbd <= 5000
   arg(4)  = '-step=1.0h'                ! or 2.1d 10.23s
   arg(5)  = '-tscale=UTC'               ! or TT
   arg(6)  = '-observer=@sun'            ! or IAU code of observer's location, e.g. 007 or 500 for geocenter
   arg(7)  = '-teph=4'                   ! 1:astrometric J2000, 2:apparent of the date, 3: mean of the date, 4: mean J2000
   arg(8)  = '-tcoor=1'                  ! 1:spherical, 2:rectangular
   arg(9)  = '-rplane=1'                 ! 1:equator, 2: ecliptic
   arg(10) = '-mime=text'                ! votable | html | text
   arg(11) = '-extrap=0'                 ! number of decimals of results
   arg(12) = '-from=MyProject'           ! Word which define the name of the caller application, or which describes the request (without space)
   ! Compose les options
   opt = trim(arg(1))
   do i = 2, size(arg)
      opt = trim(opt)//'&'//trim(arg(i))
   end do
   ! Compose la commande HTTP
   cmd = trim(wget)//' "'//trim(url)//trim(opt)//'"'
   ! Execution de la commande
   err = system(cmd)
   if (err == -1) then
      errnum = ierrno()
      print *, 'System error #',errnum
      stop
   end if
   ! Load the ephemeris
   call load_results('ephemeris.dat',nbDates,results,ticket)
   ! Display ephemeris
   do i = 1, nbDates
      if (results(i)(1:1) == '#') then
         print '(A)',trim(results(i))
      else
         read(results(i),'(2x,A22,3x,A14,1x,A14,1x,F14.9,1x,F7.2)') epoch, RA, DEC, Dist, Mv
         print '(2x,A22,3x,A14,1x,A14,1x,F14.9,1x,F7.2)',epoch,RA,DEC,Dist,Mv 
      end if
   end do
   
   contains

   !>
   !! Read the ephemeris file (fmt=text) and load the ephemeris
   !<
   subroutine load_results(file, nbd, cr, ticket)
      implicit none
      character(len=*), intent(in)       :: file !< Ephemeris input file
      integer, intent(out)               :: nbd !< Number of ephemeris
      character(len=256), dimension(100) :: cr !< Results formatted as string
      integer(8), intent(out)            :: ticket !< Ticket of the request
      
      character(len=13)                  :: cticket
      character(len=256)                 :: line, ctmp
      integer                            :: i, err, flag, idp

      ! Init.
      nbd = 0
      ticket = 0
      err = 0
      i = 1

      ! Read the file
      open(unit=10,file=trim(file),status='old',iostat=err)
      if (err /= 0) then
         print *,'File open error (unit=10) #',err
         stop
      end if
      ! Load the ephemeris
      loop: do while (.not. IS_IOSTAT_END(err))
         if (err == 0) then
            read(unit=10,fmt='(A)',iostat=err) line
            idp = verify('Flag', line)
            if (idp == 0) then
               read(line,'(A8,I2)') ctmp, flag
               cycle loop
            end if
            idp = verify('Ticket', line)
            if (idp == 0) then
               read(line,'(A10,I13)') ctmp, ticket
               cycle loop
            end if
            if (len_trim(line) > 0) then
               cr(i) = trim(line)
               i = i + 1
            end if
         else
            print *,'Read error (unit=10) #',err
            stop
         end if
      end do loop
      nbd = i-1
      close(unit=10)

      ! If flag != 1 the error
      if (flag /= 1) then
         print *,'The Miriade service sent an error, with message:'
         print *,'> ',trim(ctmp)
         stop
      end if
      
      return
   end subroutine

end program client_miriade
